<?php
/**
 * This file is used for fetching data from database.
 *
 * @author  TrulyWP
 * @package wp-migration-trulywp/includes
 * @version 3.0.1
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} //exit if accessed directly.
if ( ! is_user_logged_in() ) {
	return;
} else {
	$access_granted = false;
	if ( isset( $user_role_permission ) && count( $user_role_permission ) > 0 ) {
		foreach ( $user_role_permission as $permission ) {
			if ( current_user_can( $permission ) ) {
				$access_granted = true;
				break;
			}
		}
	}
	if ( ! $access_granted ) {
		return;
	} else {
		if ( ! function_exists( 'get_migration_trulywp_unserialize_data' ) ) {
			/**
			 * This function is used to get unserialized data.
			 *
			 * @param string $manage_data .
			 */
			function get_migration_trulywp_unserialize_data( $manage_data ) {
				$unserialize_complete_data = array();
				if ( count( $manage_data ) > 0 ) {
					foreach ( $manage_data as $value ) {
						$unserialize_data = maybe_unserialize( $value->meta_value );

						$unserialize_data['meta_id'] = $value->meta_id;
						array_push( $unserialize_complete_data, $unserialize_data );
					}
				}
				return $unserialize_complete_data;
			}
		}

		if ( ! function_exists( 'get_migration_trulywp_destinations_unserialize_data' ) ) {
			/**
			 * This function is used to get destinations unserialized data.
			 *
			 * @param string $manage_data .
			 */
			function get_migration_trulywp_destinations_unserialize_data( $manage_data ) {
				$unserialize_complete_data = array();
				if ( count( $manage_data ) > 0 ) {
					foreach ( $manage_data as $value ) {
						$unserialize_destination_data = maybe_unserialize( $value->meta_value );
						foreach ( $unserialize_destination_data as $key => $data ) {
							$unserialize_complete_data[ $key ] = $data;
						}
					}
				}
				return $unserialize_complete_data;
			}
		}

		if ( ! function_exists( 'get_migration_trulywp_tables' ) ) {
			/**
			 * This function is used to get tables.
			 *
			 * @param string $result .
			 */
			function get_migration_trulywp_tables( $result ) {
				$tables = array();
				for ( $flag = 0; $flag < count( $result ); $flag++ ) { 
					if ( migration_trulywp_restore() != $result[ $flag ]->Name ) { // WPCS: loose comparison ok.
						array_push( $tables, $result[ $flag ]->Name );
					}
				}
				return $tables;
			}
		}

		if ( ! function_exists( 'get_migration_trulywp_schedule_time' ) ) {
			/**
			 * This function is used to get schedule time.
			 *
			 * @param string $schedule_name .
			 */
			function get_migration_trulywp_schedule_time( $schedule_name ) {
				$execution_time        = '';
				$scheduler_migration_trulywp = _get_cron_array();
				if ( count( $scheduler_migration_trulywp ) > 0 ) {
					foreach ( $scheduler_migration_trulywp as $value => $key ) {
						$arr_key = array_keys( $key );
						foreach ( $arr_key as $row ) {
							if ( strstr( $row, $schedule_name ) ) {
								$execution_time = $value;
							}
						}
					}
				}
				return $execution_time;
			}
		}
		if ( isset( $_GET['page'] ) ) {
			$page = sanitize_text_field( wp_unslash( $_GET['page'] ) );// WPCS: CSRF ok,WPCS: input var ok.
		}
		$check_migration_trulywp_wizard = get_option( 'migration-trulywp-wizard' );
		$licensing_url            = false === $check_migration_trulywp_wizard ? 'twp_wizard_migration' : $page;
		if ( isset( $_GET['page'] ) ) { // WPCS: CSRF ok, input var ok.
			switch ( $licensing_url ) {
				case 'twp_roles_and_capabilities':
					$roles_data                 = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key=%s', 'roles_and_capabilities'
						)
					);// WPCS: db call ok, no-cache ok.
					$details_roles_capabilities = maybe_unserialize( $roles_data );
					$other_roles_array          = $details_roles_capabilities['capabilities'];
					break;

				case 'twp_start_migration':
					if ( is_multisite() ) {
						$name     = '';
						$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );// WPCS: db call ok, no-cache ok.
						if ( isset( $blog_ids ) && count( $blog_ids ) > 0 ) {
							foreach ( $blog_ids as $blog_id ) {
								$name .= " AND Name NOT LIKE '" . $wpdb->prefix . $blog_id . "%'";
							}
						}
						$migration_tables = 'SHOW TABLE STATUS FROM `' . DB_NAME . "` WHERE Name LIKE '" . $wpdb->prefix . "%'" . $name;
					} else {
						$migration_tables = 'SHOW TABLE STATUS FROM `' . DB_NAME . '`';
					}
					$result = $wpdb->get_results( $migration_tables );
					$result = get_migration_trulywp_tables( $result );

					$settings_data       = $wpdb->get_results(
						$wpdb->prepare(
							'SELECT * FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key in (%s,%s,%s,%s,%s,%s,%s,%s)', 'dropbox_settings', 'email_settings', 'ftp_settings', 'amazons3_settings', 'onedrive_settings', 'rackspace_settings', 'azure_settings', 'google_drive'
						)
					);// WPCS: db call ok, no-cache ok.
					$settings_data_array = get_migration_trulywp_destinations_unserialize_data( $settings_data );
					break;

				case 'twp_manage_migrations':
					$twp_migrations_id = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT id FROM ' . $wpdb->prefix . 'migration_trulywp WHERE type = %s', 'migrations'
						)
					);

					$twp_migrations_data              = $wpdb->get_results(
						$wpdb->prepare(
							'SELECT * FROM ' . $wpdb->prefix . 'migration_trulywp_meta INNER JOIN ' . $wpdb->prefix . 'migration_trulywp ON ' . $wpdb->prefix . 'migration_trulywp.id=' . $wpdb->prefix . 'migration_trulywp_meta.meta_id WHERE parent_id = %d ORDER BY ' . $wpdb->prefix . 'migration_trulywp.id desc', $twp_migrations_id
						)
					);
					$twp_migrations_unserialized_data = get_migration_trulywp_unserialize_data( $twp_migrations_data );
					break;


				case 'twp_alert_setup':
					$twp_alert_setup_updated_data = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key=%s', 'alert_setup'
						)
					);// WPCS: db call ok, no-cache ok.
					$twp_alert_setup_array        = maybe_unserialize( $twp_alert_setup_updated_data );
					break;

				case 'twp_other_settings':
					$twp_other_settings_updated_data = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key=%s', 'other_settings'
						)
					);// WPCS: db call ok, no-cache ok.
					$twp_other_settings_array        = maybe_unserialize( $twp_other_settings_updated_data );

					$twp_other_settings_maintenance_mode      = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_restore WHERE meta_key = %s', 'maintenance_mode_settings'
						)
					);// WPCS: db call ok, no-cache ok.
					$twp_other_settings_maintenance_mode_data = maybe_unserialize( $twp_other_settings_maintenance_mode );
					break;

				case 'twp_email_settings':
					$email_setting_data       = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key=%s', 'email_settings'
						)
					); // WPCS: db call ok, no-cache ok.
					$email_setting_data_array = maybe_unserialize( $email_setting_data );
					break;
			}
		}
	}
}
