<?php
/**
 * This file is used for managing data in database.
 *
 * @author  TrulyWP
 * @package wp-migration-trulywp/lib
 * @version 3.0.1
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly
if ( ! is_user_logged_in() ) {
	return;
} else {
	$access_granted = false;
	if ( isset( $user_role_permission ) && count( $user_role_permission ) > 0 ) {
		foreach ( $user_role_permission as $permission ) {
			if ( current_user_can( $permission ) ) {
				$access_granted = true;
				break;
			}
		}
	}
	if ( ! $access_granted ) {
		return;
	} else {
		if ( ! function_exists( 'get_migration_trulywp_unserialize_data' ) ) {
			/**
			 * This Function is used get data.
			 *
			 * @param string $manage_data passes parameter as manage data.
			 */
			function get_migration_trulywp_unserialize_data( $manage_data ) {
				$unserialize_complete_data = array();
				if ( count( $manage_data ) > 0 ) {
					foreach ( $manage_data as $value ) {
						$unserialize_data            = maybe_unserialize( $value->meta_value );
						$unserialize_data['meta_id'] = $value->meta_id;
						array_push( $unserialize_complete_data, $unserialize_data );
					}
				}
				return $unserialize_complete_data;
			}
		}

		if ( isset( $_REQUEST['param'] ) ) {
			$obj_dbhelper_migration_trulywp = new Dbhelper_Migration_TrulyWP();
			$dbmailer_migration_trulywp_obj = new Dbmailer_Migration_TrulyWP();
			switch ( sanitize_text_field( wp_unslash( $_REQUEST['param'] ) ) ) {// WPCS: input var ok, CSRF ok.
				case 'migration_trulywp_manual_migration_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_manual_migration' ) ) {
						parse_str( isset( $_REQUEST['data'] ) ? base64_decode( wp_unslash( filter_input( INPUT_POST, 'data' ) ) ) : '', $migration_trulywp_data_array );
						$encrypted_tables    = isset( $_REQUEST['encrypted_tables'] ) ? array_map( 'sanitize_text_field', is_array( json_decode( stripslashes( $_REQUEST['encrypted_tables'] ) ) ) ? json_decode( stripslashes( $_REQUEST['encrypted_tables'] ) ) : array() ) : array();// WPCS: input var ok, sanitization ok.
						$timezone_difference = '';
						$migrations_id          = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT id FROM ' . $wpdb->prefix . 'migration_trulywp WHERE type=%s', 'migrations'
							)
						);// WPCS: db call ok, cache ok.

						$migration_trulywp_data                          = array();
						$migration_trulywp_data['timezone_difference']   = '';
						$migration_trulywp_data['migration_name']           = 'Migration - ' . date_i18n( get_option( 'date_format' ) );
						$migration_trulywp_data['migration_type']           = sanitize_text_field( $migration_trulywp_data_array['ux_ddl_migration_type'] );
						$migration_trulywp_data['exclude_list']          = sanitize_text_field( $migration_trulywp_data_array['ux_txt_return_email'] );
						$migration_trulywp_data['file_compression_type'] = sanitize_text_field( $migration_trulywp_data_array['ux_ddl_file_compression_type'] );
						$migration_trulywp_data['db_compression_type']   = sanitize_text_field( $migration_trulywp_data_array['ux_ddl_db_compression_type'] );
						$migration_trulywp_data['migration_tables']         = sanitize_text_field( implode( ',', $encrypted_tables ) );
						$migration_trulywp_data['archive']               = isset( $_REQUEST['archive'] ) ? maybe_serialize( array( sanitize_text_field( wp_unslash( $_REQUEST['archive'] ) ) ) ) : '';
						$migration_trulywp_data['archive_name']          = isset( $_REQUEST['archive_name'] ) ? maybe_serialize( array( sanitize_text_field( wp_unslash( $_REQUEST['archive_name'] ) ) ) ) : '';
						$migration_trulywp_data['migration_destination']    = sanitize_text_field( $migration_trulywp_data_array['ux_rdl_migration_destination_type'] );
						$migration_trulywp_data['folder_location']       = sanitize_text_field( $migration_trulywp_data_array['ux_txt_content_location'] ) . sanitize_text_field( $migration_trulywp_data_array['ux_txt_folder_location'] );
						$migration_trulywp_data['execution']             = sanitize_text_field( 'manual' );
						$migration_trulywp_data['migration_urlpath']        = content_url() . sanitize_text_field( $migration_trulywp_data_array['ux_txt_folder_location'] );
						$migration_trulywp_data['log_filename']          = isset( $_REQUEST['archive_name'] ) ? maybe_serialize( array( sanitize_text_field( wp_unslash( $_REQUEST['archive_name'] ) ) . '.txt' ) ) : '';
						$migration_trulywp_data['executed_time']         = LOCAL_TIME_MIGRATION_TRULYWP;
						$migration_trulywp_data['status']                = 'running';

						$insert_manual_data              = array();
						$insert_manual_data['type']      = 'manual_migration';
						$insert_manual_data['parent_id'] = $migrations_id;
						$last_id                         = $obj_dbhelper_migration_trulywp->insert_command( migration_trulywp(), $insert_manual_data );

						$migration_trulywp_data['meta_id'] = $last_id;

						$migration_trulywp_insert_data               = array();
						$migration_trulywp_insert_data['meta_key']   = 'manual_migration_meta';// WPCS: db sql slow query.
						$migration_trulywp_insert_data['meta_value'] = maybe_serialize( $migration_trulywp_data );// WPCS: db sql slow query.
						$migration_trulywp_insert_data['meta_id']    = $last_id;
						$obj_dbhelper_migration_trulywp->insert_command( migration_trulywp_meta(), $migration_trulywp_insert_data );

						$obj_migration_data_migration_trulywp = new Migration_Data_Migration_TrulyWP();
						$obj_migration_data_migration_trulywp->close_browser_connection();
						do_action( 'start_migration', $migration_trulywp_data );
					}
					break;

				case 'migration_trulywp_change_email_template_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_change_template' ) ) {
						$template_type = isset( $_REQUEST['data'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['data'] ) ) : '';

						$email_templates_data_array = $wpdb->get_results(
							$wpdb->prepare(
								'SELECT * FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', "$template_type"
							)
						);// WPCS: db call ok, cache ok.
						$email_templates_data       = get_migration_trulywp_unserialize_data( $email_templates_data_array );
						echo wp_json_encode( $email_templates_data );
					}
					break;

				case 'migration_trulywp_other_settings_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_other_settings' ) ) {
						parse_str( isset( $_REQUEST['data'] ) ? base64_decode( wp_unslash( filter_input( INPUT_POST, 'data' ) ) ) : '', $migration_trulywp_other_settings_data );

						$twp_other_settings_id = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT id FROM ' . $wpdb->prefix . 'migration_trulywp WHERE type = %s', 'other_settings'
							)
						);// WPCS: db call ok, cache ok.

						$twp_other_settings_data                               = array();
						$twp_other_settings_data['automatic_plugin_updates']   = 'disable';
						$twp_other_settings_data['remove_tables_at_uninstall'] = sanitize_text_field( $migration_trulywp_other_settings_data['ux_ddl_remove_tables'] );
						$twp_other_settings_data['Maintenance_mode']           = sanitize_text_field( $migration_trulywp_other_settings_data['ux_ddl_maintenance_mode'] );

						$update_other_settings               = array();
						$where                               = array();
						$where['meta_id']                    = $twp_other_settings_id;
						$where['meta_key']                   = 'other_settings';// WPCS: db sql slow query.
						$update_other_settings['meta_value'] = maybe_serialize( $twp_other_settings_data );// WPCS: db sql slow query.
						$obj_dbhelper_migration_trulywp->update_command( migration_trulywp_meta(), $update_other_settings, $where );

						$twp_other_settings_maintenance_mode                         = array();
						$twp_other_settings_maintenance_mode['message_when_restore'] = sanitize_text_field( $migration_trulywp_other_settings_data['ux_txt_maintenance_mode_message'] );
						$twp_other_settings_maintenance_mode['restoring']            = sanitize_text_field( $migration_trulywp_other_settings_data['ux_ddl_maintenance_mode'] );
						$update_other_settings_maintenance_mode                     = array();
						$where             = array();
						$where['meta_key'] = 'maintenance_mode_settings';// WPCS: db sql slow query.
						$update_other_settings_maintenance_mode['meta_value'] = maybe_serialize( $twp_other_settings_maintenance_mode );// WPCS: db sql slow query.
						$obj_dbhelper_migration_trulywp->update_command( migration_trulywp_restore(), $update_other_settings_maintenance_mode, $where );
					}
					break;

				case 'migration_trulywp_email_settings_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_email_settings' ) ) {
						parse_str( isset( $_REQUEST['data'] ) ? base64_decode( wp_unslash( filter_input( INPUT_POST, 'data' ) ) ) : '', $email_settings_form_data );

						$twp_email_settings_id                         = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT id FROM ' . $wpdb->prefix . 'migration_trulywp WHERE type = %s', 'email_settings'
							)
						);// WPCS: db call ok, cache ok.
						$update_email_setting_data                    = array();
						$update_email_setting_data['migration_to_email'] = sanitize_text_field( $email_settings_form_data['ux_ddl_email_settings_enable_disable'] );
						$update_email_setting_data['email_address']   = sanitize_text_field( $email_settings_form_data['ux_txt_email_address'] );
						$update_email_setting_data['cc_email']        = sanitize_text_field( $email_settings_form_data['ux_txt_email_cc'] );
						$update_email_setting_data['bcc_email']       = sanitize_text_field( $email_settings_form_data['ux_txt_email_bcc'] );
						$update_email_setting_data['email_subject']   = sanitize_text_field( $email_settings_form_data['ux_txt_email_subject'] );
						$update_email_setting_data['email_message']   = htmlspecialchars_decode( $email_settings_form_data['ux_txt_email_settings_message'] );

						$email_setting_data               = array();
						$where                            = array();
						$where['meta_id']                 = $twp_email_settings_id;
						$where['meta_key']                = 'email_settings';// WPCS: db sql slow query.
						$email_setting_data['meta_value'] = maybe_serialize( $update_email_setting_data );// WPCS: db sql slow query.
						$obj_dbhelper_migration_trulywp->update_command( migration_trulywp_meta(), $email_setting_data, $where );
					}
					break;

				case 'migration_trulywp_manage_migrations_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_manage_migrations' ) ) {
						$migration_id    = isset( $_REQUEST['id'] ) ? intval( $_REQUEST['id'] ) : '';
						$restore_path = isset( $_REQUEST['restore_path'] ) ? sanitize_text_field( $_REQUEST['restore_path'] ) : '';// WPCS: input var ok, sanitization ok.

						$twp_migration_data          = $wpdb->get_row(
							$wpdb->prepare(
								'SELECT meta_key,meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_id = %d', $migration_id
							)
						);// WPCS: db call ok, cache ok.
						$twp_migration_data_array    = maybe_unserialize( $twp_migration_data->meta_value );
						$file_name               = basename( $restore_path );
						$obj_migration_trulywp_restore = new migration_trulywp_restore( $twp_migration_data_array, $restore_path );
						switch ( $twp_migration_data_array['migration_type'] ) {
							case 'only_database':
								$restore_ret = $obj_migration_trulywp_restore->migration_trulywp_restore_migration_db( untrailingslashit( $twp_migration_data_array['folder_location'] ) . '/' . $file_name );
								break;

							default:
								$restore_ret = $obj_migration_trulywp_restore->migration_trulywp_restore_migration();
						}

						$restore_time = $obj_migration_trulywp_restore->restore_timetaken;
						$time_taken   = round( $restore_time, 1 );
						$log_file     = $obj_migration_trulywp_restore->logfile_name;

						$migration_trulywp_data['timezone_difference']   = '';
						$migration_trulywp_data['migration_name']           = $twp_migration_data_array['migration_name'];
						$migration_trulywp_data['migration_type']           = $twp_migration_data_array['migration_type'];
						$migration_trulywp_data['exclude_list']          = $twp_migration_data_array['exclude_list'];
						$migration_trulywp_data['file_compression_type'] = $twp_migration_data_array['file_compression_type'];
						$migration_trulywp_data['db_compression_type']   = $twp_migration_data_array['db_compression_type'];
						$migration_trulywp_data['migration_tables']         = $twp_migration_data_array['migration_tables'];
						$migration_trulywp_data['archive']               = $twp_migration_data_array['archive'];
						$migration_trulywp_data['archive_name']          = $twp_migration_data_array['archive_name'];
						$migration_trulywp_data['migration_destination']    = $twp_migration_data_array['migration_destination'];
						$migration_trulywp_data['folder_location']       = $twp_migration_data_array['folder_location'];
						$migration_trulywp_data['execution']             = $twp_migration_data_array['execution'];
						$migration_trulywp_data['status']                = $restore_ret;
						$migration_trulywp_data['executed_in']           = $time_taken;
						$migration_trulywp_data['total_size']            = $twp_migration_data_array['total_size'];
						$migration_trulywp_data['meta_id']               = $twp_migration_data_array['meta_id'];
						$migration_trulywp_data['executed_time']         = LOCAL_TIME_MIGRATION_TRULYWP;
						$migration_trulywp_data['migration_urlpath']        = $twp_migration_data_array['migration_urlpath'];
						$migration_trulywp_data['execution_time']        = $twp_migration_data_array['execution_time'];
						$migration_trulywp_data['log_filename']          = $twp_migration_data_array['log_filename'];

						$log_url_filename = str_replace( str_replace( '\\', '/', untrailingslashit( ABSPATH ) ), site_url(), $log_file );

						if ( ! isset( $twp_migration_data_array['restore_log_filename'] ) ) {
							$migration_trulywp_data['restore_log_urlpath']    = maybe_serialize( array( $log_url_filename ) );
							$migration_trulywp_data['restore_log_filename']   = maybe_serialize( array( $log_file ) );
							$migration_trulywp_data['restore_execution_time'] = maybe_serialize( array( LOCAL_TIME_MIGRATION_TRULYWP ) );
						} else {
							$restore_log_urlpath    = maybe_unserialize( $twp_migration_data_array['restore_log_urlpath'] );
							$restore_execution_time = maybe_unserialize( $twp_migration_data_array['restore_execution_time'] );
							$restore_log_array      = maybe_unserialize( $twp_migration_data_array['restore_log_filename'] );
							if ( ! in_array( $log_file, $restore_log_array, true ) ) {
								$restore_log_array[]      = $log_file;
								$restore_log_urlpath[]    = $log_url_filename;
								$restore_execution_time[] = LOCAL_TIME_MIGRATION_TRULYWP;
							} else {
								$index                            = array_search( $log_file, $restore_log_array, true );
								$restore_execution_time[ $index ] = LOCAL_TIME_MIGRATION_TRULYWP;
							}
							$migration_trulywp_data['restore_log_urlpath']    = maybe_serialize( $restore_log_urlpath );
							$migration_trulywp_data['restore_log_filename']   = maybe_serialize( $restore_log_array );
							$migration_trulywp_data['restore_execution_time'] = maybe_serialize( $restore_execution_time );
						}

						echo 'restore_terminated' === $restore_ret ? '11' : '';

						$twp_restore_data = $wpdb->get_row(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_id = %d', $migration_id
							)
						);// WPCS: db call ok, cache ok.
						if ( '' !== $twp_restore_data ) {
							$manage_migrations_data               = array();
							$where                             = array();
							$where['meta_id']                  = $migration_id;
							$where['meta_key']                 = $twp_migration_data->meta_key;// WPCS: db sql slow query.
							$manage_migrations_data['meta_value'] = maybe_serialize( $migration_trulywp_data );// WPCS: db sql slow query.
							$obj_dbhelper_migration_trulywp->update_command( migration_trulywp_meta(), $manage_migrations_data, $where );
						} else {
							$restore_id = $wpdb->get_var(
								$wpdb->prepare(
									'SELECT id From ' . $wpdb->prefix . 'migration_trulywp WHERE type= %s', 'migrations'
								)
							);// WPCS: db call ok, cache ok.

							$insert_restore_data              = array();
							$insert_restore_data['type']      = 'migration_schedule_meta' === $twp_migration_data->meta_key ? 'schedule_migration' : 'manual_migration';
							$insert_restore_data['parent_id'] = $restore_id;
							$last_id                          = $obj_dbhelper_migration_trulywp->insert_command( migration_trulywp(), $insert_restore_data );

							$migration_trulywp_restore_data               = array();
							$migration_trulywp_restore_data['meta_key']   = $twp_migration_data->meta_key;// WPCS: db sql slow query.
							$migration_trulywp_restore_data['meta_value'] = maybe_serialize( $migration_trulywp_data );// WPCS: db sql slow query.
							$migration_trulywp_restore_data['meta_id']    = $last_id;
							$obj_dbhelper_migration_trulywp->insert_command( migration_trulywp_meta(), $migration_trulywp_restore_data );
						}

						$alert_setup_data       = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'alert_setup'
							)
						);// WPCS: db call ok, cache ok.
						$alert_setup_data_array = maybe_unserialize( $alert_setup_data );
						if ( 'restore_terminated' !== $restore_ret ) {
							if ( 'enable' === $alert_setup_data_array['email_when_restore_completed_successfully'] ) {
								$migration_restore_data       = $wpdb->get_var(
									$wpdb->prepare(
										'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'template_for_restore_successfully'
									)
								);// WPCS: db call ok, cache ok.
								$migration_restore_data_array = maybe_unserialize( $migration_restore_data );
								$dbmailer_migration_trulywp_obj->template_for_restore_successfully( $migration_restore_data_array, $migration_trulywp_data );
							}
						} else {
							if ( 'enable' === $alert_setup_data_array['email_when_restore_failed'] ) {
								$email_migration_restore       = $wpdb->get_var(
									$wpdb->prepare(
										'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key =%s', 'template_for_restore_failure'
									)
								);// WPCS: db call ok, cache ok.
								$email_migration_restore_array = maybe_unserialize( $email_migration_restore );
								$dbmailer_migration_trulywp_obj->template_for_restore_failure( $email_migration_restore_array, $migration_trulywp_data );
							}
						}
					}
					break;

				case 'migration_trulywp_manage_migrations_delete_module':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_manage_migrations_delete' ) ) {
						$where                 = array();
						$where_meta            = array();
						$where['id']           = isset( $_REQUEST['id'] ) ? intval( $_REQUEST['id'] ) : '';
						$twp_migration_data        = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_id = %d', $where['id']
							)
						);// WPCS: db call ok, cache ok.
						$twp_migration_data_array  = maybe_unserialize( $twp_migration_data );
						$where_meta['meta_id'] = isset( $_REQUEST['id'] ) ? intval( $_REQUEST['id'] ) : '';
						$obj_dbhelper_migration_trulywp->delete_command( migration_trulywp(), $where );
						$obj_dbhelper_migration_trulywp->delete_command( migration_trulywp_meta(), $where_meta );

						if ( 'scheduled' === $twp_migration_data_array['execution'] ) {
							$cron_name = 'migration_scheduler_' . $where['id'];
							unschedule_events_migration_trulywp( $cron_name );
						}
						$migration_archive  = maybe_unserialize( $twp_migration_data_array['archive'] );
						$logfile_archive = maybe_unserialize( $twp_migration_data_array['log_filename'] );
						if ( isset( $migration_archive ) && count( $migration_archive ) > 0 ) {
							foreach ( $migration_archive as $value ) {
								if ( 'file_exists' !== $twp_migration_data_array['status'] ) {
									@unlink( untrailingslashit( $twp_migration_data_array['folder_location'] ) . '/' . $value );
								}
							}
						}
						if ( isset( $logfile_archive ) && count( $logfile_archive ) > 0 ) {
							foreach ( $logfile_archive as $value ) {
								if ( 'file_exists' !== $twp_migration_data_array['status'] ) {
									@unlink( untrailingslashit( $twp_migration_data_array['folder_location'] ) . '/' . $value );
									@unlink( untrailingslashit( $twp_migration_data_array['folder_location'] ) . '/' . str_replace( '.txt', '.json', $value ) );
								}
							}
						}
						if ( isset( $twp_migration_data_array['restore_log_filename'] ) ) {
							$restore_logfile = maybe_unserialize( $twp_migration_data_array['restore_log_filename'] );
							if ( 'file_exists' !== $twp_migration_data_array['status'] ) {
								if ( isset( $restore_logfile ) && count( $restore_logfile ) > 0 ) {
									foreach ( $restore_logfile as $value ) {
										@unlink( $value );
										@unlink( str_replace( '.txt', '.json', $value ) );
									}
								}
							}
						}
					}
					break;

				case 'migration_trulywp_restore_message':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_restore_message' ) ) {
						$restore_path = isset( $_REQUEST['restore_path'] ) ? sanitize_text_field( $_REQUEST['restore_path'] ) : '';// WPCS: input var ok, sanitization ok.
						$location     = trailingslashit( str_replace( content_url(), WP_CONTENT_DIR, dirname( dirname( $restore_path ) ) ) ) . 'restore/';
						! is_dir( $location ) ? wp_mkdir_p( $location ) : '';

						$remove_ext   = strstr( basename( $restore_path ), '.' );
						$archive_name = str_replace( $remove_ext, '', basename( $restore_path ) );

						$file_name     = trailingslashit( $location ) . $archive_name . '.json';
						$file_url_path = trailingslashit( dirname( dirname( $restore_path ) ) ) . 'restore/' . $archive_name . '.json';

						$result = 1;
						file_put_contents( $file_name, '' );
						$message  = '{' . "\r\n";
						$message .= '"log": "Restoring Migration" ,' . "\r\n";
						$message .= '"perc": ' . $result . "\r\n";
						$message .= '}';
						file_put_contents( $file_name, $message );

						echo $file_url_path;// WPCS: XSS ok.
					}
					break;

				case 'check_cloud_connection':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_check_ftp_dropbox_connection' ) ) {
						$migration_destination = isset( $_REQUEST['migration_destination'] ) ? base64_decode( wp_unslash( $_REQUEST['migration_destination'] ) ) : '';// WPCS: input var ok, sanitization ok.
						$migration_type        = isset( $_REQUEST['type'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['type'] ) ) : '';

						$archive_name = isset( $_REQUEST['archive_name'] ) ? base64_decode( wp_unslash( $_REQUEST['archive_name'] ) ) : '';// WPCS: input var ok, sanitization ok.
						$location     = base64_decode( isset( $_REQUEST['content_location'] ) ? $_REQUEST['content_location'] : '' ) . base64_decode( isset( $_REQUEST['folder_location'] ) ? wp_unslash( $_REQUEST['folder_location'] ) : '' );// WPCS: input var ok, sanitization ok.
						! is_dir( $location ) ? wp_mkdir_p( $location ) : '';

						$file_name = trailingslashit( $location ) . $archive_name . '.json';

						$path          = trailingslashit( base64_decode( isset( $_REQUEST['folder_location'] ) ? $_REQUEST['folder_location'] : '' ) );// WPCS: input var ok, sanitization ok.
						$file_url_path = content_url() . $path . $archive_name . '.json';

						$result = 1;

						file_put_contents( $file_name, '' );
						$message  = '{' . "\r\n";
						$message .= '"log": "Checking compatibility and optimizing files" ,' . "\r\n";
						$message .= '"perc": ' . $result . ',' . "\r\n";
						$message .= '"status": "Starting" ,' . "\r\n";
						$message .= '"cloud": 1' . "\r\n";
						$message .= '}';
						file_put_contents( $file_name, $message );
						echo untrailingslashit( $file_url_path );
					}
					break;

				case 'check_cloud_connection_rerun':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_check_ftp_dropbox_connection_rerun' ) ) {
						$migration_destination = isset( $_REQUEST['migration_destination'] ) ? base64_decode( $_REQUEST['migration_destination'] ) : '';// WPCS: input var ok, sanitization ok.

						$archive_name = isset( $_REQUEST['archive_name'] ) ? base64_decode( $_REQUEST['archive_name'] ) : '';// WPCS: input var ok, sanitization ok.
						$location     = base64_decode( $_REQUEST['location'] );
						! is_dir( $location ) ? wp_mkdir_p( $location ) : '';

						$file_name     = trailingslashit( $location ) . $archive_name . '.json';
						$file_url_path = str_replace( str_replace( '\\', '/', WP_CONTENT_DIR ), content_url(), $file_name );

						$result = 1;
						$message  = '{' . "\r\n";
						$message .= '"log": "Re-Creating Migration" ,' . "\r\n";
						$message .= '"perc": ' . $result . "\r\n";
						$message .= '"cloud": 1' . "\r\n";
						$message .= '}';
						file_put_contents( $file_name, $message );
						echo $file_url_path;// WPCS: XSS ok.
					}
					break;

				case 'migration_trulywp_rerun_migrations':
					if ( wp_verify_nonce( isset( $_REQUEST['_wp_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wp_nonce'] ) ) : '', 'migration_trulywp_manage_rerun_migrations' ) ) {
						$migration_id = isset( $_REQUEST['id'] ) ? intval( $_REQUEST['id'] ) : '';

						$migrations_data       = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_id=%d', $migration_id
							)
						);// WPCS: db call ok, cache ok.
						$migrations_data_array = maybe_unserialize( $migrations_data );

						@unlink( untrailingslashit( $migrations_data_array['folder_location'] ) . '/' . implode( '', maybe_unserialize( $migrations_data_array['archive'] ) ) );
						@unlink( untrailingslashit( $migrations_data_array['folder_location'] ) . '/' . implode( '', maybe_unserialize( $migrations_data_array['log_filename'] ) ) );

						$migrations_data_array['timezone_difference'] = '';
						if ( isset( $migrations_data_array['old_migration'] ) ) {
							unset( $migrations_data_array['old_migration'] );
							unset( $migrations_data_array['old_migration_logfile'] );
						}
						$obj_migration_data_migration_trulywp = new Migration_Data_Migration_TrulyWP();
						$obj_migration_data_migration_trulywp->close_browser_connection();
						do_action( 'start_migration', $migrations_data_array );
					}
					break;

			}
			die();
		}
	}
}
