<?php
//templated and files
if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly
if ( ! is_user_logged_in() ) {
	return;
} else {
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	} else {
		if ( ! class_exists( 'DbHelper_Install_Script_Migration_TrulyWP' ) ) {
			/**
			 * This Class is used for Insert Update operations.
			 */
			class DbHelper_Install_Script_Migration_TrulyWP {
				/**
				 * This Function is used for Insert data in database.
				 *
				 * @param string $table_name .
				 * @param array  $data .
				 */
				public function insert_command( $table_name, $data ) {
					global $wpdb;
					$wpdb->insert( $table_name, $data );
					return $wpdb->insert_id;
				}

				/**
				 * This function is used for Update data.
				 *
				 * @param string $table_name .
				 * @param array  $data .
				 * @param string $where .
				 */
				public function update_command( $table_name, $data, $where ) {
					global $wpdb;
					$wpdb->update( $table_name, $data, $where );
				}
			}
		}

		if ( ! function_exists( 'migration_trulywp_table' ) ) {
			/**
			 * This function is used to create parent table.
			 */
			function migration_trulywp_table() {
				global $wpdb;
				$obj_dbhelper_migration_trulywp_parent = new DbHelper_Install_Script_Migration_TrulyWP();
				$sql                             = 'CREATE TABLE IF NOT EXISTS ' . migration_trulywp() . '
				(
					`id` int(11) NOT NULL AUTO_INCREMENT,
					`type` varchar(100) NOT NULL,
					`parent_id` int(11) NOT NULL,
					PRIMARY KEY (`id`)
				)
				ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1';
				dbDelta( $sql );

				$data = 'INSERT INTO ' . migration_trulywp() . " (`type`, `parent_id`) VALUES
				('migrations', 0),
				('general_settings', 0),
				('email_templates', 0),
				('collation_type', 0),
				('roles_and_capabilities', 0)";
				dbDelta( $data );

				$migration_trulywp_parent_table = $wpdb->get_results(
					'SELECT * FROM ' . $wpdb->prefix . 'migration_trulywp'
				);
				if ( isset( $migration_trulywp_parent_table ) && count( $migration_trulywp_parent_table ) > 0 ) {
					foreach ( $migration_trulywp_parent_table as $row ) {
						switch ( $row->type ) {
							case 'general_settings':
								$general_settings                       = array();
								$general_settings['alert_setup']        = $row->id;
								$general_settings['other_settings']     = $row->id;
								$general_settings['dropbox_settings']   = $row->id;
								$general_settings['email_settings']     = $row->id;
								$general_settings['ftp_settings']       = $row->id;
								$general_settings['amazons3_settings']  = $row->id;
								$general_settings['onedrive_settings']  = $row->id;
								$general_settings['rackspace_settings'] = $row->id;
								$general_settings['azure_settings']     = $row->id;
								$general_settings['google_drive']       = $row->id;

								foreach ( $general_settings as $key => $value ) {
									$general_settings_data              = array();
									$general_settings_data['type']      = $key;
									$general_settings_data['parent_id'] = $value;
									$obj_dbhelper_migration_trulywp_parent->insert_command( migration_trulywp(), $general_settings_data );
								}
								break;
						}
					}
				}
			}
		}

		if ( ! function_exists( 'migration_trulywp_meta_table' ) ) {
			/**
			 * This function is used to create meta table.
			 */
			function migration_trulywp_meta_table() {
				$obj_dbhelper_install_script_migration_trulywp = new DbHelper_Install_Script_Migration_TrulyWP();
				global $wpdb;
				$sql = 'CREATE TABLE IF NOT EXISTS ' . migration_trulywp_meta() . '
				(
					`id` int(11) NOT NULL AUTO_INCREMENT,
					`meta_id` int(11) NOT NULL,
					`meta_key` varchar(255) NOT NULL,
					`meta_value` longtext NOT NULL,
					PRIMARY KEY (`id`)
				)
				ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1';
				dbDelta( $sql );

				$admin_email = get_option( 'admin_email' );
				$admin_name  = get_option( 'blogname' );

				$migration_trulywp_table_data = $wpdb->get_results(
					'SELECT * FROM ' . $wpdb->prefix . 'migration_trulywp'
				);
				if ( isset( $migration_trulywp_table_data ) && count( $migration_trulywp_table_data ) > 0 ) {
					foreach ( $migration_trulywp_table_data as $row ) {
						switch ( $row->type ) {
							case 'alert_setup':
								$alert_setup_data_array = array();
								$alert_setup_data_array['email_when_migration_scheduled_successfully']  = 'enable';
								$alert_setup_data_array['email_when_migration_generated_successfully']  = 'enable';
								$alert_setup_data_array['email_when_migration_failed']                  = 'enable';
								$alert_setup_data_array['email_when_restore_completed_successfully'] = 'enable';
								$alert_setup_data_array['email_when_restore_failed']                 = 'enable';

								$alert_setup_data               = array();
								$alert_setup_data['meta_id']    = $row->id;
								$alert_setup_data['meta_key']   = 'alert_setup';
								$alert_setup_data['meta_value'] = maybe_serialize( $alert_setup_data_array );
								$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_meta(), $alert_setup_data );
								break;

							case 'other_settings':
								$other_settings_data_array                               = array();
								$other_settings_data_array['automatic_plugin_updates']   = 'disable';
								$other_settings_data_array['remove_tables_at_uninstall'] = 'enable';
								$other_settings_data_array['Maintenance_mode']           = 'disable';

								$other_settings_data               = array();
								$other_settings_data['meta_id']    = $row->id;
								$other_settings_data['meta_key']   = 'other_settings';
								$other_settings_data['meta_value'] = maybe_serialize( $other_settings_data_array );
								$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_meta(), $other_settings_data );
								break;

							case 'email_settings':
								$email_settings_data_array                    = array();
								$email_settings_data_array['migration_to_email'] = 'disable';
								$email_settings_data_array['email_address']   = $admin_email;
								$email_settings_data_array['cc_email']        = '';
								$email_settings_data_array['bcc_email']       = '';
								$email_settings_data_array['email_subject']   = '[migration_type] Successfully Generated - Migration TrulyWP';
								$email_settings_data_array['email_message']   = '<p>Hi Admin,</p><p>Kindly find attached Compressed Migration for <strong>[migration_type]</strong> in <strong>[compression_type]</strong> Format with Detailed Log executed by <strong>[username]</strong> on <strong>[start_time]</strong> for your website <strong>[site_url]</strong>.</p><p><u>Here are the details for the Migration :-</u></p><p><strong>Archive Name: </strong>[archive_name]</p><p><strong>Migration Name</strong>: [migration_name]</p><p><strong>Migration Type:</strong> [migration_type]</p><p><strong>Exclude List</strong>: [exclude_list]</p><p><strong>Compression Type</strong>: [compression_type]</p><p><strong>Migration Tables</strong>: [migration_tables]</p><p>Thank you for using <strong>Migration TrulyWP</strong> Plugin.</p><p><strong>Thanks & Regards</strong></p><p><strong>Support Team</strong><br /> <strong>TrulyWP<br /> </strong></p>';

								$email_settings_data               = array();
								$email_settings_data['meta_id']    = $row->id;
								$email_settings_data['meta_key']   = 'email_settings';
								$email_settings_data['meta_value'] = maybe_serialize( $email_settings_data_array );
								$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_meta(), $email_settings_data );
								break;

							case 'email_templates':
								$email_templates = array();
								$email_templates['template_for_migration_successful_generated'] = '<p>Hi,</p><p>A Migration File has been Successfully Generated for your website <strong>[site_url]</strong> by <strong>[username] </strong>at <strong>[start_time]</strong>.</p><p><u>Here are the Details for the Migration :-</u></p><p><strong>Migration Name: </strong>[migration_name]</p><p><strong>Migration Type:</strong> [migration_type]</p><p><strong>Exclude List:</strong> [exclude_list]</p><p><strong>File Compression Type:</strong> [file_compression_type]</p><p><strong>DB Compression Type:</strong> [db_compression_type]</p><p><strong>Migration Tables:</strong> [migration_tables]</p><p><strong>Archive Name:</strong> [archive_name]</p><p><strong>Migration Destination:</strong> [migration_destination]</p><p><strong>Folder Location:</strong> [folder_location]</p><p><strong>Thanks & Regards</strong></p><p><strong>Support Team</strong><br /> <strong>TrulyWP<br /> </strong></p>';
								$email_templates['template_for_migration_failure']              = '<p>Hi,</p><p>A Migration has been Failed to Generate to your website <strong>[site_url]</strong> by <strong>[username]</strong> at <strong>[start_time]</strong>.</p><p><u>Here are the Details for the Migration :-</u></p><p><strong>Migration Name: </strong>[migration_name]</p><p><strong>Migration Type:</strong> [migration_type]</p><p><strong>Exclude List:</strong> [exclude_list]</p><p><strong>File Compression Type:</strong> [file_compression_type]</p><p><strong>DB Compression Type:</strong> [db_compression_type]</p><p><strong>Migration Tables:</strong> [migration_tables]</p><p><strong>Archive Name:</strong> [archive_name]</p><p><strong>Migration Destination:</strong> [migration_destination]</p><p><strong>Folder Location:</strong> [folder_location]</p><p><strong>Thanks & Regards</strong></p><p><strong>Support Team</strong><br /> <strong>TrulyWP<br /> </strong></p>';
								
								$email_templates_message = array( 'Migration Successfully Generated Notification - Migration TrulyWP', 'Migration Successfully Scheduled Notification - Migration TrulyWP', 'Migration Restore Success Notification - Migration TrulyWP', 'Migration Failure Notification - Migration TrulyWP', 'Migration Restore Failure Notification - Migration TrulyWP' );
								$count                   = 0;
								foreach ( $email_templates as $key => $value ) {
									$email_templates_scheduled_migration_array                  = array();
									$email_templates_scheduled_migration_array['email_send_to'] = $admin_email;
									$email_templates_scheduled_migration_array['email_cc']      = '';
									$email_templates_scheduled_migration_array['email_bcc']     = '';
									$email_templates_scheduled_migration_array['email_subject'] = $email_templates_message[ $count ];
									$email_templates_scheduled_migration_array['email_message'] = $value;
									$count++;

									$email_templates_for_scheduled_migration               = array();
									$email_templates_for_scheduled_migration['meta_id']    = $row->id;
									$email_templates_for_scheduled_migration['meta_key']   = $key;
									$email_templates_for_scheduled_migration['meta_value'] = maybe_serialize( $email_templates_scheduled_migration_array );
									$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_meta(), $email_templates_for_scheduled_migration );
								}
								break;

							case 'roles_and_capabilities':
								$roles_capabilities_data_array                                   = array();
								$roles_capabilities_data_array['roles_and_capabilities']         = '1,1,1,0,0,0';
								$roles_capabilities_data_array['show_migration_trulywp_top_bar_menu']  = 'enable';
								$roles_capabilities_data_array['administrator_privileges']       = '1,1,1,1,1,1,1,1,1';
								$roles_capabilities_data_array['author_privileges']              = '0,1,0,0,0,1,0,0,0';
								$roles_capabilities_data_array['editor_privileges']              = '0,1,1,0,0,1,0,1,0';
								$roles_capabilities_data_array['contributor_privileges']         = '0,0,0,0,0,1,0,0,0';
								$roles_capabilities_data_array['subscriber_privileges']          = '0,0,0,0,0,0,0,0,0';
								$roles_capabilities_data_array['others_full_control_capability'] = '0';
								$roles_capabilities_data_array['other_privileges']               = '0,0,0,0,0,0,0,0,0';

								$user_capabilities        = get_others_capabilities_migration_trulywp();
								$other_roles_array        = array();
								$other_roles_access_array = array(
									'manage_options',
									'edit_plugins',
									'edit_posts',
									'publish_posts',
									'publish_pages',
									'edit_pages',
									'read',
								);
								foreach ( $other_roles_access_array as $role ) {
									if ( in_array( $role, $user_capabilities, true ) ) {
										array_push( $other_roles_array, $role );
									}
								}
								$roles_capabilities_data_array['capabilities'] = $other_roles_array;

								$roles_data_array               = array();
								$roles_data_array['meta_id']    = $row->id;
								$roles_data_array['meta_key']   = 'roles_and_capabilities';
								$roles_data_array['meta_value'] = maybe_serialize( $roles_capabilities_data_array );
								$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_meta(), $roles_data_array );
								break;
						}
					}
				}
			}
		}


		if ( ! function_exists( 'migration_trulywp_table_restore' ) ) {
			/**
			 * This function is used to create restore table.
			 */
			function migration_trulywp_table_restore() {
				global $wpdb;
				$obj_dbhelper_install_script_migration_trulywp = new DbHelper_Install_Script_Migration_TrulyWP();
				$sql                                     = 'CREATE TABLE IF NOT EXISTS ' . migration_trulywp_restore() . '
				(
					`id` int(11) NOT NULL AUTO_INCREMENT,
					`meta_key` varchar(100) NOT NULL,
					`meta_value` longtext,
					PRIMARY KEY (`id`)
				)
				ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1';
				dbDelta( $sql );

				$maintenance_mode_settings                         = array();
				$maintenance_mode_settings['message_when_restore'] = 'Site in Maintenance Mode';
				$maintenance_mode_settings['restoring']            = 'disable';

				$maintenance_mode_settings_data               = array();
				$maintenance_mode_settings_data['meta_key']   = 'maintenance_mode_settings';
				$maintenance_mode_settings_data['meta_value'] = maybe_serialize( $maintenance_mode_settings );
				$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp_restore(), $maintenance_mode_settings_data );
			}
		}

		$obj_dbhelper_install_script_migration_trulywp = new DbHelper_Install_Script_Migration_TrulyWP();
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		$migration_trulywp_version_number = get_option( 'migration-trulywp-version-number' );

		switch ( $migration_trulywp_version_number ) {
			case '':
				migration_trulywp_table();
				migration_trulywp_meta_table();
				migration_trulywp_table_restore();
				$migration_trulywp_admin_notices_array                    = array();
				$twp_start_date                                      = date( 'm/d/Y' );
				$twp_start_date                                      = strtotime( $twp_start_date );
				$twp_start_date                                      = strtotime( '+7 day', $twp_start_date );
				$twp_start_date                                      = date( 'm/d/Y', $twp_start_date );
				$migration_trulywp_admin_notices_array['two_week_review'] = array( 'start' => $twp_start_date, 'int' => 7, 'dismissed' => 0 ); 
				update_option( 'twp_admin_notice', $migration_trulywp_admin_notices_array );
				break;

			default:
				if ( $wpdb->query( "SHOW TABLES LIKE '" . $wpdb->prefix . 'migration_details' . "'" ) !== 0 && $wpdb->query( "SHOW TABLES LIKE '" . $wpdb->prefix . 'migration_meta' . "'" ) !== 0 ) {
					$wpdb->query( 'DROP TABLE IF EXISTS ' . $wpdb->prefix . 'migration_details' );
					$wpdb->query( 'DROP TABLE IF EXISTS ' . $wpdb->prefix . 'migration_meta' );
					migration_trulywp_table();
					migration_trulywp_meta_table();
					migration_trulywp_table_restore();
				}
				if ( $wpdb->query( "SHOW TABLES LIKE '" . $wpdb->prefix . 'migration_trulywp' . "'" ) !== 0 && $wpdb->query( "SHOW TABLES LIKE '" . $wpdb->prefix . 'migration_trulywp_meta' . "'" ) !== 0 ) {
					$twp_other_settings_updated_data = $wpdb->get_var(
						$wpdb->prepare(
							'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key=%s', 'other_settings'
						)
					);
					$twp_other_settings_array        = maybe_unserialize( $twp_other_settings_updated_data );
					if ( ! array_key_exists( 'automatic_plugin_updates', $twp_other_settings_array ) ) {
						$twp_other_settings_array['automatic_plugin_updates'] = 'disable';
					}
					$where             = array();
					$where['meta_key'] = 'other_settings';
					$twp_other_settings_serialized_data['meta_value'] = maybe_serialize( $twp_other_settings_array );
					$obj_dbhelper_install_script_migration_trulywp->update_command( migration_trulywp_meta(), $twp_other_settings_serialized_data, $where );
				}
				$get_collate_status_data = $wpdb->query(
					$wpdb->prepare(
						'SELECT type FROM ' . $wpdb->prefix . 'migration_trulywp WHERE type=%s', 'collation_type'
					)
				);// db call ok; no-cache ok.
				$charset_collate         = '';
				if ( ! empty( $wpdb->charset ) ) {
					$charset_collate .= 'CONVERT TO CHARACTER SET ' . $wpdb->charset;
				}
				if ( ! empty( $wpdb->collate ) ) {
					$charset_collate .= ' COLLATE ' . $wpdb->collate;
				}
				if ( 0 === $get_collate_status_data ) {
					if ( ! empty( $charset_collate ) ) {
						$change_collate_main_table         = $wpdb->query(
							'ALTER TABLE ' . $wpdb->prefix . 'migration_trulywp ' . $charset_collate 
						);
						$change_collate_meta_table         = $wpdb->query(
							'ALTER TABLE ' . $wpdb->prefix . 'migration_trulywp_meta ' . $charset_collate 
						);
						$change_collate_restore_table      = $wpdb->query(
							'ALTER TABLE ' . $wpdb->prefix . 'migration_trulywp_restore ' . $charset_collate 
						);
						$collation_data_array              = array();
						$collation_data_array['type']      = 'collation_type';
						$collation_data_array['parent_id'] = '0';
						$obj_dbhelper_install_script_migration_trulywp->insert_command( migration_trulywp(), $collation_data_array );
					}
				}
				break;
		}
		update_option( 'migration-trulywp-version-number', '1.0.0' );
	}
}
