<?php
// Email class 
if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly
if ( ! class_exists( 'Dbmailer_Migration_TrulyWP' ) ) {
	/**
	 * This Class is used for send Emails.
	 */
	class Dbmailer_Migration_TrulyWP {
		// This function is used for sending Emails when migration is successfully generated.
		public function email_when_migration_generated_successfully( $migration_generated_data_array, $migration_trulywp_data ) {
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
				include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
			}
			$datetime = date_i18n( 'd M Y H:i:s' );
			global $current_user;
			$headers  = '';
			$headers .= 'Content-Type: text/html; charset= utf-8' . "\r\n";
			if ( '' !== $migration_generated_data_array['email_cc'] ) {
				$headers .= 'Cc: ' . $migration_generated_data_array['email_cc'] . "\r\n";
			}
			if ( '' !== $migration_generated_data_array['email_bcc'] ) {
				$headers .= 'Bcc: ' . $migration_generated_data_array['email_bcc'] . "\r\n";
			}
			switch ( $migration_trulywp_data['migration_type'] ) {
				case 'only_themes':
					$type             = $twp_themes;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins':
					$type             = $twp_plugins;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_wp_content_folder':
					$type             = $twp_contents;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'complete_migration':
					$type           = $twp_complete_migration;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression = $migration_trulywp_data['db_compression_type'];
					$db_tables      = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table  .= '</ul>';
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_filesystem':
					$type             = $twp_filesystem;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins_and_themes':
					$type             = $twp_plugins_themes;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_database':
					$type           = $twp_database;
					$exclude        = $twp_na;
					$db_compression = $migration_trulywp_data['db_compression_type'];
					$db_tables      = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table  .= '</ul>';
					$file_compression = $twp_na;
					break;
			}

			$migration_dest = $twp_local_folder;

			$subject         = $migration_generated_data_array['email_subject'];
			$replace_subject = str_replace( '[migration_type]', $type, $subject );
			$message         = '<div style="font-family: Calibri;">';
			$message        .= $migration_generated_data_array['email_message'];
			$message        .= '</div>';

			$archive_name = implode( '', maybe_unserialize( $migration_trulywp_data['archive'] ) );
			$replace_type_message            = str_replace( '[migration_type]', $type, $message );
			$replace_type_site_url           = str_replace( '[site_url]', site_url(), $replace_type_message );
			$replace_type_archive            = str_replace( '[archive_name]', '<a href=' . $migration_trulywp_data['migration_urlpath'] . $archive_name . '>' . $archive_name . '</a>', $replace_type_site_url );
			$replace_type_migration_name        = str_replace( '[migration_name]', $migration_trulywp_data['migration_name'], $replace_type_archive );
			$replace_type_exclude_list       = str_replace( '[exclude_list]', $exclude, $replace_type_migration_name );
			$replace_type_file_compression   = str_replace( '[file_compression_type]', $file_compression, $replace_type_exclude_list );
			$replace_type_db_compression     = str_replace( '[db_compression_type]', $db_compression, $replace_type_file_compression );
			$replace_type_table              = str_replace( '[migration_tables]', $database_table, $replace_type_db_compression );
			$replace_type_location           = str_replace( '[folder_location]', $migration_trulywp_data['folder_location'], $replace_type_table );
			$replace_type_start_time         = str_replace( '[start_time]', $datetime, $replace_type_location );
			$user                            = 'scheduled' === $migration_trulywp_data['execution'] ? $twp_scheduler : $current_user->display_name;
			$replace_type_username           = str_replace( '[username]', $user, $replace_type_start_time );
			$replace_type_migration_destination = str_replace( '[migration_destination]', $migration_dest, $replace_type_username );

			wp_mail( $migration_generated_data_array['email_send_to'], $replace_subject, $replace_type_migration_destination, $headers );
		}
		/**
		 * This function is used for sending Emails when migration is failed.
		 *
		 * @param array $email_migration_data_array .
		 * @param array $migration_trulywp_data .
		 */
		public function email_when_migration_failed( $email_migration_data_array, $migration_trulywp_data ) {
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
				include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
			}
			$start_time = date_i18n( 'd M Y H:i:s' );
			global $current_user;

			$headers  = '';
			$headers .= 'Content-Type: text/html; charset= utf-8' . "\r\n";
			if ( '' !== $email_migration_data_array['email_cc'] ) {
				$headers .= 'Cc: ' . $email_migration_data_array['email_cc'] . "\r\n";
			}
			if ( '' !== $email_migration_data_array['email_bcc'] ) {
				$headers .= 'Bcc: ' . $email_migration_data_array['email_bcc'] . "\r\n";
			}
			switch ( $migration_trulywp_data['migration_type'] ) {
				case 'only_themes':
					$type             = $twp_themes;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins':
					$type             = $twp_plugins;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_wp_content_folder':
					$type             = $twp_contents;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'complete_migration':
					$type           = $twp_complete_migration;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression = $migration_trulywp_data['db_compression_type'];
					$db_tables      = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table  .= '</ul>';
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_filesystem':
					$type             = $twp_filesystem;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins_and_themes':
					$type             = $twp_plugins_themes;
					$exclude          = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_compression   = $twp_na;
					$database_table   = $twp_na;
					$file_compression = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_database':
					$type           = $twp_database;
					$exclude        = $twp_na;
					$db_compression = $migration_trulywp_data['db_compression_type'];
					$db_tables      = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table  .= '</ul>';
					$file_compression = $twp_na;
					break;
			}

			$migration_destination = $twp_local_folder;

			$subject         = $email_migration_data_array['email_subject'];
			$replace_subject = str_replace( '[migration_type]', $type, $subject );
			$message         = '<div style="font-family: Calibri;">';
			$message        .= $email_migration_data_array['email_message'];
			$message        .= '</div>';
			$migration_site_url = str_replace( '[site_url]', site_url(), $message );

			$migration_archive_name     = str_replace( '[archive_name]', $twp_na, $migration_site_url );
			$migration_name             = str_replace( '[migration_name]', $migration_trulywp_data['migration_name'], $migration_archive_name );
			$migration_exclude_list     = str_replace( '[exclude_list]', $exclude, $migration_name );
			$migration_file_compression = str_replace( '[file_compression_type]', $file_compression, $migration_exclude_list );
			$migration_db_compression   = str_replace( '[db_compression_type]', $db_compression, $migration_file_compression );
			$migration_table            = str_replace( '[migration_tables]', $database_table, $migration_db_compression );
			$migration_location         = str_replace( '[folder_location]', $migration_trulywp_data['folder_location'], $migration_table );
			$user                    = 'scheduled' === $migration_trulywp_data['execution'] ? $twp_scheduler : $current_user->display_name;
			$migration_current_user     = str_replace( '[username]', $user, $migration_location );
			$start_date_time         = str_replace( '[start_time]', $start_time, $migration_current_user );
			$destination             = str_replace( '[migration_destination]', $migration_destination, $start_date_time );
			$replace_migration_type     = str_replace( '[migration_type]', $type, $destination );

			wp_mail( $email_migration_data_array['email_send_to'], $replace_subject, $replace_migration_type, $headers );
		}

		/**
		 * This function is used for sending migration to Email.
		 */
		public function sending_migration_to_email( $email_settings_array, $migration_trulywp_data ) {
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
				include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
			}
			$start_time = date_i18n( 'd M Y H:i:s' );
			global $current_user;
			$headers  = '';
			$headers .= 'Content-Type: text/html; charset= utf-8' . "\r\n";
			if ( '' !== $email_settings_array['cc_email'] ) {
				$headers .= 'Cc: ' . $email_settings_array['cc_email'] . "\r\n";
			}
			if ( '' !== $email_settings_array['bcc_email'] ) {
				$headers .= 'Bcc: ' . $email_settings_array['bcc_email'] . "\r\n";
			}

			switch ( $migration_trulywp_data['migration_type'] ) {
				case 'only_themes':
					$type           = $twp_themes;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$database_table = $twp_na;
					$compression    = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins':
					$type           = $twp_plugins;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_tables      = $twp_na;
					$database_table = $twp_na;
					$compression    = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_wp_content_folder':
					$type           = $twp_contents;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$database_table = $twp_na;
					$compression    = $migration_trulywp_data['file_compression_type'];
					break;

				case 'complete_migration':
					$type      = $twp_complete_migration;
					$exclude   = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$db_tables = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table .= '</ul>';
					$compression     = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_filesystem':
					$type           = $twp_filesystem;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$database_table = $twp_na;
					$compression    = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_plugins_and_themes':
					$type           = $twp_plugins_themes;
					$exclude        = '' !== $migration_trulywp_data['exclude_list'] ? $migration_trulywp_data['exclude_list'] : $twp_na;
					$database_table = $twp_na;
					$compression    = $migration_trulywp_data['file_compression_type'];
					break;

				case 'only_database':
					$type           = $twp_database;
					$exclude        = $twp_na;
					$db_compression = $migration_trulywp_data['db_compression_type'];
					$db_tables      = $migration_trulywp_data['migration_tables'];

					$data_table_array = explode( ',', $db_tables );
					$database_table   = '<ul>';
					if ( isset( $data_table_array ) && count( $data_table_array ) > 0 ) {
						foreach ( $data_table_array as $row ) {
							$database_table .= '<li style="margin-left: 0px;">' . $row . '</li>';
						}
					}
					$database_table .= '</ul>';
					$compression     = $migration_trulywp_data['db_compression_type'];
					break;
			}

			$subject         = $email_settings_array['email_subject'];
			$replace_subject = str_replace( '[migration_type]', $type, $subject );
			$message         = '<div style="font-family: Calibri;">';
			$message        .= $email_settings_array['email_message'];
			$message        .= '</div>';

			if ( 'scheduled' === $migration_trulywp_data['execution'] ) {
				$archive_array = maybe_unserialize( $migration_trulywp_data['archive'] );
				$archive_name  = $archive_array[ count( $archive_array ) - 2 ];
				$logfile_array = maybe_unserialize( $migration_trulywp_data['log_filename'] );
				$logfile_name  = $logfile_array[ count( $logfile_array ) - 2 ];
			} else {
				$archive_name = implode( '', maybe_unserialize( $migration_trulywp_data['archive'] ) );
				$logfile_name = implode( '', maybe_unserialize( $migration_trulywp_data['log_filename'] ) );
			}
			$replace_type_message        = str_replace( '[migration_type]', $type, $message );
			$replace_type_site_url       = str_replace( '[site_url]', site_url(), $replace_type_message );
			$replace_type_archive        = str_replace( '[archive_name]', '<a href=' . $migration_trulywp_data['migration_urlpath'] . $archive_name . '>' . $archive_name . '</a>', $replace_type_site_url );
			$replace_type_migration_name    = str_replace( '[migration_name]', $migration_trulywp_data['migration_name'], $replace_type_archive );
			$replace_type_exclude_list   = str_replace( '[exclude_list]', $exclude, $replace_type_migration_name );
			$replace_type_db_compression = str_replace( '[compression_type]', $compression, $replace_type_exclude_list );
			$replace_type_table          = str_replace( '[migration_tables]', $database_table, $replace_type_db_compression );
			$user                        = 'scheduled' === $migration_trulywp_data['execution'] ? $twp_scheduler : $current_user->display_name;
			$migration_current_user         = str_replace( '[username]', $user, $replace_type_table );
			$start_date_time             = str_replace( '[start_time]', $start_time, $migration_current_user );

			if ( filesize( untrailingslashit( $migration_trulywp_data['folder_location'] ) . '/' . $archive_name ) <= 20971520 ) {
				$attachment = array( untrailingslashit( $migration_trulywp_data['folder_location'] ) . '/' . $archive_name, untrailingslashit( $migration_trulywp_data['folder_location'] ) . '/' . $logfile_name );
				wp_mail( $email_settings_array['email_address'], $replace_subject, $start_date_time, $headers, $attachment );
			}
		}

	}
}
