<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly.
if ( ! is_user_logged_in() ) {
	return;
} else {
	$access_granted = false;
	if ( isset( $user_role_permission ) && count( $user_role_permission ) > 0 ) {
		foreach ( $user_role_permission as $permission ) {
			if ( current_user_can( $permission ) ) {
				$access_granted = true;
				break;
			}
		}
	}
	if ( ! $access_granted ) {
		return;
	} else {
		$flag = 0;

		$role_capabilities = $wpdb->get_var(
			$wpdb->prepare(
				'SELECT meta_value from ' . $wpdb->prefix . 'migration_trulywp_meta WHERE ' . $wpdb->prefix . 'migration_trulywp_meta.meta_key = %s', 'roles_and_capabilities'
			)
		);// WPCS: db call ok, no-cache ok.

		$roles_and_capabilities_unserialized_data = maybe_unserialize( $role_capabilities );
		$capabilities                             = explode( ',', $roles_and_capabilities_unserialized_data['roles_and_capabilities'] );

		if ( is_super_admin() ) {
			$twp_role = 'administrator';
		} else {
			$twp_role = check_user_roles_migration_trulywp();
		}
		switch ( $twp_role ) {
			case 'administrator':
				$privileges = 'administrator_privileges';
				$flag       = $capabilities[0];
				break;

			case 'author':
				$privileges = 'author_privileges';
				$flag       = $capabilities[1];
				break;

			case 'editor':
				$privileges = 'editor_privileges';
				$flag       = $capabilities[2];
				break;

			case 'contributor':
				$privileges = 'contributor_privileges';
				$flag       = $capabilities[3];
				break;

			case 'subscriber':
				$privileges = 'subscriber_privileges';
				$flag       = $capabilities[4];
				break;

			default:
				$privileges = 'other_privileges';
				$flag       = $capabilities[5];
				break;
		}

		if ( isset( $roles_and_capabilities_unserialized_data ) && count( $roles_and_capabilities_unserialized_data ) > 0 ) {
			foreach ( $roles_and_capabilities_unserialized_data as $key => $value ) {
				if ( $key === $privileges ) {
					$privileges_value = $value;
					break;
				}
			}
		}

		$full_control = explode( ',', $privileges_value );
		if ( ! defined( 'FULL_CONTROL' ) ) {
			define( 'FULL_CONTROL', "$full_control[0]" );
		}
		if ( ! defined( 'MANAGE_MIGRATIONS_MIGRATION_TRULYWP' ) ) {
			define( 'MANAGE_MIGRATIONS_MIGRATION_TRULYWP', "$full_control[1]" );
		}
		if ( ! defined( 'MANUAL_MIGRATION_TRULYWP' ) ) {
			define( 'MANUAL_MIGRATION_TRULYWP', "$full_control[2]" );
		}
		if ( ! defined( 'SCHEDULE_MIGRATION_TRULYWP' ) ) {
			define( 'SCHEDULE_MIGRATION_TRULYWP', "$full_control[3]" );
		}
		if ( ! defined( 'GENERAL_SETTINGS_MIGRATION_TRULYWP' ) ) {
			define( 'GENERAL_SETTINGS_MIGRATION_TRULYWP', "$full_control[4]" );
		}
		if ( ! defined( 'EMAIL_TEMPLATES_MIGRATION_TRULYWP' ) ) {
			define( 'EMAIL_TEMPLATES_MIGRATION_TRULYWP', "$full_control[5]" );
		}
		if ( ! defined( 'ROLES_AND_CAPABILITIES_MIGRATION_TRULYWP' ) ) {
			define( 'ROLES_AND_CAPABILITIES_MIGRATION_TRULYWP', "$full_control[6]" );
		}
		if ( ! defined( 'SYSTEM_INFORMATION_MIGRATION_TRULYWP' ) ) {
			define( 'SYSTEM_INFORMATION_MIGRATION_TRULYWP', "$full_control[7]" );
		}

		$migration_trulywp_wizard_url = get_option( 'migration-trulywp-wizard' );
		if ( '1' === $flag ) {
			global $wp_version;
			
			add_menu_page( 'TrulyWP Migration', 'TrulyWP Migration', 'read', 'twp_start_migration', '', plugins_url( 'assets/global/img/icon.png', dirname( __FILE__ ) ) );
			
			add_submenu_page( 'twp_start_migration', 'Generate File', 'Generate File', 'read', 'twp_start_migration', 'twp_start_migration' );
			add_submenu_page( 'twp_start_migration', 'Manage Files', 'Manage Files', 'read', 'twp_manage_migrations',  'twp_manage_migrations' );
			
		}

		if ( ! function_exists( 'twp_start_migration' ) ) {
			/**
			 * This function is used to create twp_start_migration menu.
			 */
			function twp_start_migration() {
				global $wpdb;
				$user_role_permission = get_users_capabilities_migration_trulywp();
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
					include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/header.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/header.php';
				}
				
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/queries.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/queries.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'views/migrations/start-migration.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'views/migrations/start-migration.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/footer.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/footer.php';
				}
			}
		}
		if ( ! function_exists( 'twp_manage_migrations' ) ) {
			/**
			 * This function is used to create twp_manage_migrations menu.
			 */
			function twp_manage_migrations() {
				global $wpdb;
				$user_role_permission = get_users_capabilities_migration_trulywp();
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
					include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/header.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/header.php';
				}
				
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/queries.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/queries.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'views/migrations/manage-migrations.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'views/migrations/manage-migrations.php';
				}
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/footer.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'includes/footer.php';
				}
			}
		}
	}
}
