<?php 
/**
 * Plugin Name: Easy Migration
 * Plugin URI: https://trulywp.com
 * Description: Migration Plugin from Truluywp
 * Author: TrulyWP
 * Author URI: https://trulywp.com
 * Version: 1.0.0
 * License: GPLv3
 * Text Domain: wp-migration-trulywp
 * Domain Path: /languages
 *
 * @package wp-migration-trulywp
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly
$current_year  = date( 'Y' );
$current_month = date( 'm' );
$current_date  = date( 'd' );

// Version Control - For now its disabled 1.0.0
//update_option( 'migration-trulywp-version-number', '1.0.0' );

/* Constant Declaration */
if ( ! defined( 'MIGRATION_TRULYWP_DIR_PATH' ) ) {
	define( 'MIGRATION_TRULYWP_DIR_PATH', plugin_dir_path( __FILE__ ) );
}
if ( ! defined( 'MIGRATION_TRULYWP_CONTENT_DIR' ) ) {
	define( 'MIGRATION_TRULYWP_CONTENT_DIR', dirname( dirname( MIGRATION_TRULYWP_DIR_PATH ) ) );
}
if ( ! defined( 'MIGRATION_TRULYWP_MIGRATIONS_DIR' ) ) {
	define( 'MIGRATION_TRULYWP_MIGRATIONS_DIR', MIGRATION_TRULYWP_CONTENT_DIR . '/wp-migration-trulywp' );
}
if ( ! defined( 'MIGRATION_TRULYWP_MIGRATIONS_YEAR_DIR' ) ) {
	define( 'MIGRATION_TRULYWP_MIGRATIONS_YEAR_DIR', MIGRATION_TRULYWP_MIGRATIONS_DIR . '/' . $current_year );
}
if ( ! defined( 'MIGRATION_TRULYWP_MIGRATIONS_MONTH_DIR' ) ) {
	define( 'MIGRATION_TRULYWP_MIGRATIONS_MONTH_DIR', MIGRATION_TRULYWP_MIGRATIONS_YEAR_DIR . '/' . $current_month );
}
if ( ! defined( 'MIGRATION_TRULYWP_MIGRATIONS_DATE_DIR' ) ) {
	define( 'MIGRATION_TRULYWP_MIGRATIONS_DATE_DIR', MIGRATION_TRULYWP_MIGRATIONS_MONTH_DIR . '/' . $current_date );
}
if ( ! defined( 'MIGRATION_TRULYWP_URL_PATH' ) ) {
	define( 'MIGRATION_TRULYWP_URL_PATH', plugins_url( __FILE__ ) );
}
if ( ! defined( 'TWP_PASSWORD_HASH' ) ) {
	define('TWP_PASSWORD_HASH','$2y$10$8Sw21CtSL13HkKD4bD0R9eORH/Odd1HsFEGuDOiCfWEWKVn1lVbqq');
}
if ( ! defined( 'MIGRATION_TRULYWP_PLUGIN_DIRNAME' ) ) {
	define( 'MIGRATION_TRULYWP_PLUGIN_DIRNAME', plugin_basename( dirname( __FILE__ ) ) );
}
if ( ! defined( 'WP_MIGRATION_TRULYWP' ) ) {
	define( 'WP_MIGRATION_TRULYWP', 'wp-migration-trulywp' );
}
if ( ! defined( 'MIGRATION_TRULYWP_FOLDER_DROPBOX' ) ) {
	define( 'MIGRATION_TRULYWP_FOLDER_DROPBOX', 'wp-migration-trulywp/' . $current_year . '/' . $current_month . '/' . $current_date . '/' );
}
if ( ! defined( 'MIGRATION_TRULYWP_SET_TIME_LIMIT' ) ) {
	define( 'MIGRATION_TRULYWP_SET_TIME_LIMIT', 0 );
}
if ( ! defined( 'MIGRATION_TRULYWP_WARN_DB_ROWS' ) ) {
	define( 'MIGRATION_TRULYWP_WARN_DB_ROWS', 150000 );
}
if ( ! defined( 'MIGRATION_TRULYWP_WARN_FILE_SIZE' ) ) {
	define( 'MIGRATION_TRULYWP_WARN_FILE_SIZE', 1024 * 1024 * 250 );
}
if ( ! defined( 'MIGRATION_TRULYWP_FILE' ) ) {
	define( 'MIGRATION_TRULYWP_FILE', plugin_basename( __FILE__ ) );
}

if ( ! defined( 'LOCAL_TIME_MIGRATION_TRULYWP' ) ) {
	define( 'LOCAL_TIME_MIGRATION_TRULYWP', strtotime( date_i18n( 'Y-m-d H:i:s' ) ) );
}
if ( ! defined( 'MIGRATION_TRULYWP_VERSION_NUMBER' ) ) {
	define( 'MIGRATION_TRULYWP_VERSION_NUMBER', '1.0.0' );
}

if ( ! function_exists( 'migration_folders_for_migration_trulywp' ) ) {
	/**
	 * This Function is used to make a folder.
	 */
	function migration_folders_for_migration_trulywp() {
		if ( ! is_dir( MIGRATION_TRULYWP_MIGRATIONS_DIR ) ) {
			wp_mkdir_p( MIGRATION_TRULYWP_MIGRATIONS_DIR );
		}
	}
}

$memory_limit_migration_trulywp = intval( ini_get( 'memory_limit' ) );
if ( ! extension_loaded( 'suhosin' ) && $memory_limit_migration_trulywp < 512 ) {
	@ini_set( 'memory_limit', '512M' );
}

@ini_set( 'max_execution_time', 0 );
@ini_set( 'max_input_vars', 10000 );

if ( ! function_exists( 'install_script_for_migration_trulywp' ) ) {
	/**
	 * This function is used to create tables in database.
	 */
	function install_script_for_migration_trulywp() {
		global $wpdb;
		update_option( 'migration-trulywp-admin-email', get_option( 'admin_email' ) );
		update_option( 'migration-trulywp-wizard', 'skip' );
					
		if ( is_multisite() ) {
			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				$version = get_option( 'migration-trulywp-version-number' );
				if ( $version < '1.0.0' ) {
					if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-install-script-migration-trulywp.php' ) ) {
						include MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-install-script-migration-trulywp.php';
					}
				}
				restore_current_blog();
			}
		} else {
			$version = get_option( 'migration-trulywp-version-number' );
			if ( $version < '1.0.0' ) {
				if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-install-script-migration-trulywp.php' ) ) {
					include_once MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-install-script-migration-trulywp.php';
				}
			}
		}

		$data_for_redirect = get_option('twp_migration_setupman');
		if(!$data_for_redirect)
		{
			update_option('twp_migration_setupman', 'one');
			$random_wp = wp_rand('1000','10000');
			wp_redirect(admin_url("admin.php?page=twp_start_migration&twp_migrate_id=trulywp&".$random_wp."=".$random_wp));
			exit;
		}
		
	}
}

register_activation_hook( __FILE__, 'install_script_for_migration_trulywp_activate' );

if ( ! function_exists( 'install_script_for_migration_trulywp_activate' ) ) {
	/**
	 * This function is used to create tables in database.
	 */
	function install_script_for_migration_trulywp_activate() {
		global $wpdb;
		//Set the wizard and Skip for Future purpose
		update_option( 'migration-trulywp-admin-email', get_option( 'admin_email' ) );
		update_option( 'migration-trulywp-wizard', 'skip' );
		update_option('twp_migration_setupman', '');
		//deactivate_plugins( array( '/autoptimize/autoptimize.php', '/wp-backup-bank/wp-backup-bank.php' , '/wp-fastest-cache/wpFastestCache.php','/w3-total-cache/w3-total-cache.php','/wordfence/wordfence.php','force-https-littlebizzy/force-https.php' ) );	
	}
}
if ( ! function_exists( 'migration_trulywp' ) ) {
	/**
	 * This function is used for creating parent table.
	 */
	function migration_trulywp() {
		global $wpdb;
		return $wpdb->prefix . 'migration_trulywp';
	}
}

if ( ! function_exists( 'migration_trulywp_restore' ) ) {
	function migration_trulywp_restore() {
		global $wpdb;
		return $wpdb->prefix . 'migration_trulywp_restore';
	}
}

if ( ! function_exists( 'migration_trulywp_meta' ) ) {
	function migration_trulywp_meta() {
		global $wpdb;
		return $wpdb->prefix . 'migration_trulywp_meta';
	}
}

/**
 * This function is used for checking roles of different users.
 */

if ( ! function_exists( 'check_user_roles_migration_trulywp' ) ) {
	/**
	 * This function is used for checking roles of different users.
	 *
	 * @param string $user passes parameter as user.
	 */
	function check_user_roles_migration_trulywp( $user = null ) {
		$user = $user ? new WP_User( $user ) : wp_get_current_user();
		return $user->roles ? $user->roles[0] : false;
	}
}


if ( ! function_exists( 'get_others_capabilities_migration_trulywp' ) ) {
	/**
	 * This function is used to get all the roles available in WordPress.
	 */
	function get_others_capabilities_migration_trulywp() {
		$user_capabilities = array();
		if ( function_exists( 'get_editable_roles' ) ) {
			foreach ( get_editable_roles() as $role_name => $role_info ) {
				foreach ( $role_info['capabilities'] as $capability => $values ) {
					if ( ! in_array( $capability, $user_capabilities, true ) ) {
						array_push( $user_capabilities, $capability );
					}
				}
			}
		} else {
			$user_capabilities = array(
				'manage_options',
				'edit_plugins',
				'edit_posts',
				'publish_posts',
				'publish_pages',
				'edit_pages',
				'read',
			);
		}
		return $user_capabilities;
	}
}

$migration_trulywp_version_number = get_option( 'migration-trulywp-version-number' );
if ( '1.0.0' === $migration_trulywp_version_number ) {

	if ( is_admin() ) {
		if ( ! function_exists( 'backend_js_css_for_migration_trulywp' ) ) {
			/**
			 * This function is used for including backend js and css.
			 */
			function backend_js_css_for_migration_trulywp() {
				$pages_migration_trulywp = array(
					'twp_manage_migrations',
					'twp_start_migration',
					
				);
				if ( in_array( isset( $_REQUEST['page'] ) ? esc_attr( wp_unslash( $_REQUEST['page'] ) ) : '', $pages_migration_trulywp, true ) ) {// WPCS: input var ok, CSRF ok, sanitization ok.
					wp_enqueue_script( 'jquery' );
					wp_enqueue_script( 'jquery-ui-datepicker' );
					wp_enqueue_script( 'bootstrap.js', plugins_url( 'assets/global/plugins/custom/js/custom.js', __FILE__ ) );
					wp_enqueue_script( 'bootstrap-tabdrop.js', plugins_url( 'assets/global/plugins/tabdrop/js/tabdrop.js', __FILE__ ) );
					wp_enqueue_script( 'jquery.validate.js', plugins_url( 'assets/global/plugins/validation/jquery.validate.js', __FILE__ ) );
					wp_enqueue_script( 'jquery.datatables.js', plugins_url( 'assets/global/plugins/datatables/media/js/jquery.datatables.js', __FILE__ ) );
					wp_enqueue_script( 'jquery.fngetfilterednodes.js', plugins_url( 'assets/global/plugins/datatables/media/js/fngetfilterednodes.js', __FILE__ ) );
					wp_enqueue_script( 'toastr.js', plugins_url( 'assets/global/plugins/toastr/toastr.js', __FILE__ ) );
					wp_enqueue_style( 'simple-line-icons.css', plugins_url( 'assets/global/plugins/icons/icons.css', __FILE__ ) );
					wp_enqueue_style( 'components.css', plugins_url( 'assets/global/css/components.css', __FILE__ ) );
					wp_enqueue_style( 'wp-migration-trulywp-custom.css', plugins_url( 'assets/admin/layout/css/wp-migration-trulywp-custom.css', __FILE__ ) );
					if ( is_rtl() ) {
						wp_enqueue_style( 'migration-trulywp-bootstrap.css', plugins_url( 'assets/global/plugins/custom/css/custom-rtl.css', __FILE__ ) );
						wp_enqueue_style( 'migration-trulywp-layout.css', plugins_url( 'assets/admin/layout/css/layout-rtl.css', __FILE__ ) );
						wp_enqueue_style( 'wp-migration-trulywp-tech-migrator-custom.css', plugins_url( 'assets/admin/layout/css/tech-migrator-custom-rtl.css', __FILE__ ) );
					} else {
						wp_enqueue_style( 'migration-trulywp-bootstrap.css', plugins_url( 'assets/global/plugins/custom/css/custom.css', __FILE__ ) );
						wp_enqueue_style( 'migration-trulywp-layout.css', plugins_url( 'assets/admin/layout/css/layout.css', __FILE__ ) );
						wp_enqueue_style( 'wp-migration-trulywp-tech-migrator-custom.css', plugins_url( 'assets/admin/layout/css/tech-migrator-custom.css', __FILE__ ) );
					}
					wp_enqueue_style( 'migration-trulywp-plugins.css', plugins_url( 'assets/global/css/plugins.css', __FILE__ ) );
					wp_enqueue_style( 'migration-trulywp-default.css', plugins_url( 'assets/admin/layout/css/themes/default.css', __FILE__ ) );
					wp_enqueue_style( 'migration-trulywp-toastr.min.css', plugins_url( 'assets/global/plugins/toastr/toastr.css', __FILE__ ) );
					wp_enqueue_style( 'migration-trulywp-jquery-ui.css', plugins_url( 'assets/global/plugins/datepicker/jquery-ui.css', __FILE__ ), false, '2.0', false );
					wp_enqueue_style( 'migration-trulywp-datatables.foundation.css', plugins_url( 'assets/global/plugins/datatables/media/css/datatables.foundation.css', __FILE__ ) );
				}
			}
		}
		add_action( 'admin_enqueue_scripts', 'backend_js_css_for_migration_trulywp' );
	}

	if ( ! function_exists( 'get_users_capabilities_migration_trulywp' ) ) {
		/**
		 * This function is used to get users capabilities.
		 */
		function get_users_capabilities_migration_trulywp() {
			global $wpdb;
			$capabilities              = $wpdb->get_var(
				$wpdb->prepare(
					'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'roles_and_capabilities'
				)
			);
			$core_roles                = array(
				'manage_options',
				'edit_plugins',
				'edit_posts',
				'publish_posts',
				'publish_pages',
				'edit_pages',
				'read',
			);
			$unserialized_capabilities = maybe_unserialize( $capabilities );
			return isset( $unserialized_capabilities['capabilities'] ) ? $unserialized_capabilities['capabilities'] : $core_roles;
		}
	}

	if ( ! function_exists( 'helper_file_for_migration_trulywp' ) ) {
		/**
		 * This function is used for helper file.
		 */
		function helper_file_for_migration_trulywp() {
			global $wpdb;
			$user_role_permission = get_users_capabilities_migration_trulywp();
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-migration-trulywp.php' ) ) {
				include_once MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbhelper-migration-trulywp.php';
			}
		}
	}

	if ( ! function_exists( 'sidebar_menu_for_migration_trulywp' ) ) {
		/**
		 * This function is used for sidebar menu.
		 */
		function sidebar_menu_for_migration_trulywp() {
			global $wpdb, $current_user;
			$user_role_permission = get_users_capabilities_migration_trulywp();
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
				include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
			}
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/sidebar-menu.php' ) ) {
				include_once MIGRATION_TRULYWP_DIR_PATH . 'lib/sidebar-menu.php';
			}
		}
	}


	if ( ! function_exists( 'ajax_register_for_migration_trulywp' ) ) {
		/**
		 * This function is used for register ajax.
		 */
		function ajax_register_for_migration_trulywp() {
			global $wpdb;
			$user_role_permission = get_users_capabilities_migration_trulywp();
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php' ) ) {
				include MIGRATION_TRULYWP_DIR_PATH . 'includes/translations.php';
			}
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/action-library.php' ) ) {
				include_once MIGRATION_TRULYWP_DIR_PATH . 'lib/action-library.php';
			}
		}
	}

	if ( ! function_exists( 'plugin_load_textdomain_migration_trulywp' ) ) {
		/**
		 * This function is used to load languages.
		 */
		function plugin_load_textdomain_migration_trulywp() {
			if ( function_exists( 'load_plugin_textdomain' ) ) {
				load_plugin_textdomain( 'wp-migration-trulywp', false, MIGRATION_TRULYWP_PLUGIN_DIRNAME . '/languages' );
			}
		}
	}

	if ( ! function_exists( 'mailer_file_migration_trulywp' ) ) {
		/**
		 * This function is used for include mailer class.
		 */
		function mailer_file_migration_trulywp() {
			if ( file_exists( MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbmailer-migration-trulywp.php' ) ) {
				include_once MIGRATION_TRULYWP_DIR_PATH . 'lib/class-dbmailer-migration-trulywp.php';
			}
		}
	}

	if ( ! function_exists( 'scheduler_for_migration_trulywp' ) ) {
		/**
		 * This function is used for creating a scheduler for migration.
		 *
		 * @param string $cron_name passes parameter as cron name.
		 * @param string $time_interval passes parameter as time interval.
		 * @param string $timestamp passes parameter as time stamp.
		 */
		function scheduler_for_migration_trulywp( $cron_name, $time_interval, $timestamp ) {
			if ( ! wp_next_scheduled( $cron_name ) ) {
				$current_offset = get_option( 'gmt_offset' ) * 60 * 60;

				wp_schedule_event( $timestamp - $current_offset, $time_interval, $cron_name );
			}
		}
	}

	if ( ! function_exists( 'cron_scheduler_for_intervals_migration_trulywp' ) ) {
		/**
		 * This function is used to cron scheduler for intervals.
		 *
		 * @param string $schedules passes parameter as schedules.
		 */
		function cron_scheduler_for_intervals_migration_trulywp( $schedules ) {
			$schedules['1Hour']  = array(
				'interval' => 60 * 60,
				'display'  => 'Every 1 Hour',
			);
			$schedules['2Hour']  = array(
				'interval' => 60 * 60 * 2,
				'display'  => 'Every 2 Hours',
			);
			$schedules['3Hour']  = array(
				'interval' => 60 * 60 * 3,
				'display'  => 'Every 3 Hours',
			);
			$schedules['4Hour']  = array(
				'interval' => 60 * 60 * 4,
				'display'  => 'Every 4 Hours',
			);
			$schedules['5Hour']  = array(
				'interval' => 60 * 60 * 5,
				'display'  => 'Every 5 Hours',
			);
			$schedules['6Hour']  = array(
				'interval' => 60 * 60 * 6,
				'display'  => 'Every 6 Hours',
			);
			$schedules['7Hour']  = array(
				'interval' => 60 * 60 * 7,
				'display'  => 'Every 7 Hours',
			);
			$schedules['8Hour']  = array(
				'interval' => 60 * 60 * 8,
				'display'  => 'Every 8 Hours',
			);
			$schedules['9Hour']  = array(
				'interval' => 60 * 60 * 9,
				'display'  => 'Every 9 Hours',
			);
			$schedules['10Hour'] = array(
				'interval' => 60 * 60 * 10,
				'display'  => 'Every 10 Hours',
			);
			$schedules['11Hour'] = array(
				'interval' => 60 * 60 * 11,
				'display'  => 'Every 11 Hours',
			);
			$schedules['12Hour'] = array(
				'interval' => 60 * 60 * 12,
				'display'  => 'Every 12 Hours',
			);
			$schedules['13Hour'] = array(
				'interval' => 60 * 60 * 13,
				'display'  => 'Every 13 Hours',
			);
			$schedules['14Hour'] = array(
				'interval' => 60 * 60 * 14,
				'display'  => 'Every 14 Hours',
			);
			$schedules['15Hour'] = array(
				'interval' => 60 * 60 * 15,
				'display'  => 'Every 15 Hours',
			);
			$schedules['16Hour'] = array(
				'interval' => 60 * 60 * 16,
				'display'  => 'Every 16 Hours',
			);
			$schedules['17Hour'] = array(
				'interval' => 60 * 60 * 17,
				'display'  => 'Every 17 Hours',
			);
			$schedules['18Hour'] = array(
				'interval' => 60 * 60 * 18,
				'display'  => 'Every 18 Hours',
			);
			$schedules['19Hour'] = array(
				'interval' => 60 * 60 * 19,
				'display'  => 'Every 19 Hours',
			);
			$schedules['20Hour'] = array(
				'interval' => 60 * 60 * 20,
				'display'  => 'Every 20 Hours',
			);
			$schedules['21Hour'] = array(
				'interval' => 60 * 60 * 21,
				'display'  => 'Every 21 Hours',
			);
			$schedules['22Hour'] = array(
				'interval' => 60 * 60 * 22,
				'display'  => 'Every 22 Hours',
			);
			$schedules['23Hour'] = array(
				'interval' => 60 * 60 * 23,
				'display'  => 'Every 23 Hours',
			);
			$schedules['Daily']  = array(
				'interval' => 60 * 60 * 24,
				'display'  => 'Daily',
			);
			return $schedules;
		}
	}

	if ( ! function_exists( 'unschedule_events_migration_trulywp' ) ) {
		/**
		 * This function is used to unscheduling the events.
		 *
		 * @param string $cron_name passes parameter as cron name.
		 */
		function unschedule_events_migration_trulywp( $cron_name ) {
			if ( wp_next_scheduled( $cron_name ) ) {
				$db_cron = wp_next_scheduled( $cron_name );
				wp_unschedule_event( $db_cron, $cron_name );
			}
		}
	}

	if ( ! function_exists( 'admin_functions_for_migration_trulywp' ) ) {
		/**
		 * This function is used for admin functions.
		 */
		function admin_functions_for_migration_trulywp() {
			install_script_for_migration_trulywp();
			helper_file_for_migration_trulywp();
			migration_folders_for_migration_trulywp();
		}
	}

	if ( ! function_exists( 'user_function_for_migration_trulywp' ) ) {
		/**
		 * This function is used for user functions.
		 */
		function user_function_for_migration_trulywp() {
			plugin_load_textdomain_migration_trulywp();
			migration_folders_for_migration_trulywp();
			global $wpdb;
			$meta_values      = $wpdb->get_var(
				$wpdb->prepare(
					'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'other_settings'
				)
			);
			$meta_data_array  = array();
			$unserialize_data = maybe_unserialize( $meta_values );
		
			wp_clear_scheduled_hook( 'automatic_updates_migration_trulywp' );
		
			mailer_file_migration_trulywp();
		}
	}

	if ( ! function_exists( 'maintenance_mode_migration_trulywp' ) ) {
		function maintenance_mode_migration_trulywp() {
			//feature purpose for local to server migration
		}
	}
	if ( ! function_exists( 'migration_data_migration_trulywp' ) ) {
		/**
		 * This function is used to Running Migration.
		 *
		 * @param string $migration_array passes parameter as migration array.
		 */
		function migration_data_migration_trulywp( $migration_array ) {
			try {

				global $wpdb;
				$obj_migration_data_migration_trulywp = new Migration_Data_Migration_TrulyWP( $migration_array );

				if ( 'only_database' === $migration_array['migration_type'] ) {
					$migration_status = $obj_migration_data_migration_trulywp->database_migration_trulywp();
				} else {
					$migration_status = $obj_migration_data_migration_trulywp->get_directories_migration_trulywp();
				}

				$migration_trulywp_data     = $migration_array;
				$file_name            = untrailingslashit( $migration_trulywp_data['folder_location'] ) . '/' . implode( '', maybe_unserialize( $migration_trulywp_data['archive_name'] ) ) . '.json';
				$migration_size          = $obj_migration_data_migration_trulywp->kbsize;
				$migration_data_time     = $obj_migration_data_migration_trulywp->timetaken;
				$migration_log_timetaken = $obj_migration_data_migration_trulywp->log_timetaken;

				$migration_trulywp_data['execution_time'] = maybe_serialize( array( LOCAL_TIME_MIGRATION_TRULYWP ) );
				$migration_trulywp_data['executed_in']    = $migration_data_time;
				$migration_trulywp_data['total_size']     = $migration_size . 'Mb';
				$migration_trulywp_data['executed_time']  = LOCAL_TIME_MIGRATION_TRULYWP;

				switch ( $migration_trulywp_data['migration_type'] ) {
					case 'only_themes':
						$migration_filename = 'Themes';
						break;

					case 'only_plugins':
						$migration_filename = 'Plugins';
						break;

					case 'only_wp_content_folder':
						$migration_filename = 'Contents';
						break;

					case 'complete_migration':
						$migration_filename = 'Complete';
						break;

					case 'only_filesystem':
						$migration_filename = 'Filesystem';
						break;

					case 'only_plugins_and_themes':
						$migration_filename = 'Plugins_Themes';
						break;

					case 'only_database':
						$migration_filename = 'Database';
						break;
				}

				$dbmailer_migration_trulywp_obj = new Dbmailer_Migration_TrulyWP();

				$migration_trulywp_data['status'] = $migration_status;
				
				$migration_trulywp_update_data               = array();
				$where                                 = array();
				$where['meta_key']                     = 'manual_migration_meta';
				$migration_trulywp_update_data['meta_value'] = maybe_serialize( $migration_trulywp_data );
				$where['meta_id']                      = $migration_trulywp_data['meta_id'];
				$obj_dbhelper_migration_trulywp              = new Dbhelper_Migration_TrulyWP();
				$obj_dbhelper_migration_trulywp->update_command( migration_trulywp_meta(), $migration_trulywp_update_data, $where );

				$alert_setup_data = $wpdb->get_var(
					$wpdb->prepare(
						'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'alert_setup'
					)
				);

				$alert_setup_data_array = maybe_unserialize( $alert_setup_data );

				if ( 'terminated' !== $migration_status && 'file_exists' !== $migration_status ) {
					update_option('twp_migration_running_status','done');
					update_option('twp_migration_zip_name',implode( '', maybe_unserialize( $migration_trulywp_data['archive_name'] ) ));
					update_option('twp_migration_filename',str_replace(WP_CONTENT_DIR , content_url(), untrailingslashit( $migration_trulywp_data['folder_location'] ) . '/' . implode( '', maybe_unserialize( $migration_trulywp_data['archive_name'] ) ) . '.tar'));
										
					if ( 'enable' === $alert_setup_data_array['email_when_migration_generated_successfully'] ) {
						$migration_generated_data       = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . 'migration_trulywp_meta WHERE meta_key = %s', 'template_for_migration_successful_generated'
							)
						);
						$migration_generated_data_array = maybe_unserialize( $migration_generated_data );
						$dbmailer_migration_trulywp_obj->email_when_migration_generated_successfully( $migration_generated_data_array, $migration_trulywp_data );
					}
				} else {
					if ( 'enable' === $alert_setup_data_array['email_when_migration_failed'] ) {
						$email_migration_data       = $wpdb->get_var(
							$wpdb->prepare(
								'SELECT meta_value FROM ' . $wpdb->prefix . ' migration_trulywp_meta WHERE meta_key =%s', 'template_for_migration_failure'
							)
						);
						$email_migration_data_array = maybe_unserialize( $email_migration_data );
						$dbmailer_migration_trulywp_obj->email_when_migration_failed( $email_migration_data_array, $migration_trulywp_data );
					}
				}
			}
			catch (Exception $e)
			{
				$this->migration_trulywp_log( $e->getMessage() . "\r\n" );
			}
		}
	}

	if ( ! function_exists( 'deactivation_function_for_migration_trulywp' ) ) {
		/**
		 * This function is used to get Plugin Stats.
		 */
		function deactivation_function_for_migration_trulywp() {
			delete_option( 'migration-trulywp-wizard' );
		}
	}
	/* hooks */

	/**
	 *This hook is used for calling the function of admin functions.
	 */

	add_action( 'admin_init', 'admin_functions_for_migration_trulywp' );

	/**
	 * This hook is used for calling the function of register ajax.
	 */

	add_action( 'wp_ajax_migration_trulywp_action', 'ajax_register_for_migration_trulywp' );

	/*
	 * This hook is used for calling the function of user function.
	 */

	add_action( 'init', 'user_function_for_migration_trulywp' );

	/**
	 * This hook is used for calling the function of sidebar menu.
	 */

	add_action( 'admin_menu', 'sidebar_menu_for_migration_trulywp' );

	/**
	 * This hook is used for calling the function of sidebar menuin multisite case.
	 */
	add_action( 'network_admin_menu', 'sidebar_menu_for_migration_trulywp' );

	/**
	 * This hook is used for calling the function of cron schedulers jobs for WordPress data and database.
	 */

	add_filter( 'cron_schedules', 'cron_scheduler_for_intervals_migration_trulywp' );

	/**
	 * This hook is used for maintenance_mode_migration_trulywp.
	 */

	add_action( 'template_redirect', 'maintenance_mode_migration_trulywp' );

	/**
	 * This hook is used for start_migration.
	 */

	add_action( 'start_migration', 'migration_data_migration_trulywp' );

	
	/**
	 * This hook is used for Plugin Stats.
	 */

	register_deactivation_hook( __FILE__, 'deactivation_function_for_migration_trulywp' );
}

/**
 * This hook is used for calling the function of install script.
 */

add_action( 'admin_init', 'install_script_for_migration_trulywp' );

function create_migration_atumatically_twp()
{
	if(isset($_GET['twp_migrate_id']) && !empty($_GET['twp_migrate_id']))
	{
		$Migration_id = sanitize_text_field($_GET['twp_migrate_id']);
	    if(password_verify($Migration_id,TWP_PASSWORD_HASH))
	    {
	    	if(isset($_GET['twp_action']) && $_GET['twp_action'] == 'get_status')
			{
				$backup_status = get_option('twp_migration_running_status','no');
				if($backup_status == 'done')
				{
					// Here the file path will transfer to the server
					$get_downlaod_path = get_option('twp_migration_filename');
					$get_downlaod_folder_name = get_option('twp_migration_zip_name');
					echo json_encode( array("status" => "success","file_path" => $get_downlaod_path, "folder_name" => $get_downlaod_folder_name) );
					exit;

				}else if($backup_status == 'terminated')
				{
					echo json_encode( array("status" => "faliure","file_path" => "none") );
					exit;
				}else
				{
					// wait command will pass to server
					echo json_encode( array("status" => "wait","file_path" => "none") );
					exit;
				}
			}
			else if(isset($_GET['twp_action']) && $_GET['twp_action'] == "update_status" && isset($_GET['status']))
			{
				update_option('twp_running_migration',$_GET['status']);
				return json_encode("success");
				
			}
			else if(isset($_GET['twp_action']) && $_GET['twp_action'] == "finished")
			{
				update_option('twp_migration_running_status','finished');
				update_option('twp_migration_filename','none');
				require_once (ABSPATH . 'wp-includes/class-phpass.php');
				require_once(ABSPATH.'wp-admin/includes/user.php');

				$user = get_user_by( 'login', 'trulywp_clone_admin' );
				if ( $user ) { // get_user_by can return false, if no such user exists
				    wp_delete_user( $user->ID );
				}
				
			}
		}	
	}
}
//This is for create a Migration by automatic login
add_action( 'init', 'create_migration_atumatically_twp' );
